const electron = window.require('electron');
const ipcRenderer = electron.ipcRenderer;

import React, { useState} from 'react';
import { Context } from '../context-provider';
import { ConvertByteToHumanReadable } from '../helpers/AppHelpers';
import '../styles/Collections.css';

import LinearProgress from '@material-ui/core/LinearProgress';
import CloudDownload from '@material-ui/icons/CloudDownload';

function Collections(){
    const [ view, setView ] = useState({section:'download',type:'all'});    
    return (
        <div id="collections-container">
            <CollectionsSidebar 
                view={view} 
                onSetView={(newView) => setView(newView)} 
            />
            <CollectionsSwitchView view={view} />
        </div>
    )
}

function CollectionsSidebar(props){

    const { ocsApiState, ocsApiDispatch } = React.useContext(Context);

    function onSetView(view){
        props.onSetView(view);
    }

    let installedTypesMenuDisplay;
    if (ocsApiState.installedItemTypes && ocsApiState.installedItemTypes.length > 0){

        const InstalledTypesMenu = ocsApiState.installedItemTypes.sort().map((iit,index) => {
            const countItems = ocsApiState.installedItems.filter((i) => i.install_type === iit).length;
            const menuItemCssClass = props.view.section === "installed" && props.view.type === iit ? "selected" : "";
            return (
                <li key={index}>
                    <a className={menuItemCssClass} onClick={() => onSetView({section:'installed',type:iit})}>
                        <span className="name">{iit}</span>
                        <span className="app-badge low-emphasis">{countItems}</span>
                    </a>
                </li>
            )
        });

        installedTypesMenuDisplay = (
            <nav>
                <h4>Installed</h4>
                <ul>{InstalledTypesMenu}</ul>
            </nav>
        )

    }

    let downloadItemsCounterDisplay;
    if (ocsApiState.downloadItems.length > 0){
        const filteredDownloadItems = ocsApiState.downloadItems.filter((di) => di.status === "Downloading");
        if (filteredDownloadItems.length > 0){
            downloadItemsCounterDisplay = <span className="app-badge medium-emphasis">{filteredDownloadItems.length}</span>
        }
    }

    return (
        <aside>
            <ul className="task">
                <li>
                    <a className={props.view.section === "download" ? "selected" : ""} onClick={() => onSetView({section:'download',type:'all'})}>
                        <span className="name">Download</span>
                        {downloadItemsCounterDisplay}
                    </a>
                </li>
            </ul>
            {installedTypesMenuDisplay}
        </aside>
    )
}

function CollectionsSwitchView(props){

    const { ocsApiState } = React.useContext(Context);

    const arrayName = props.view.section + "Items";
    let items = ocsApiState[arrayName];
    if (props.view.type !== "all") items = items.filter((item) => item.install_type === props.view.type);

    const collectionitemsListDisplay = items.map((item,index) => (
        <CollectionItem key={index} item={item} section={props.view.section}/>
    ));

    return (
        <main id="collections-switch-view">{collectionitemsListDisplay}</main>
    )
}

function CollectionItem(props){

    const item = props.item;

    let collectionItemDisplay;
    if (props.section === "download"){
        collectionItemDisplay = <CollectionDownloadItem item={item} />
    } else {
        collectionItemDisplay = item.files.map((file,index) => (
            <CollectionItemFile key={index} item={item} file={file}/>
        ))    
    }
    
    return (
        <ul className="item-sublist">
            {collectionItemDisplay}
        </ul>
    )

}

function CollectionDownloadItem(props){

    const item = props.item;
    
    let progressBarDisplay, messageDisplay = "Downloading"
    if (item.status === "Downloading"){
        if (item.bytesRecieved && item.bytesTotal){
            const downloadedPercent = item.bytesRecieved / item.bytesTotal;
            progressBarDisplay = <div className="progress-bar-container"><progress data-progress value={downloadedPercent} max="1"/></div>
            messageDisplay = "Downloading... " + ConvertByteToHumanReadable(item.bytesRecieved) + "/" + ConvertByteToHumanReadable(item.bytesTotal);
        }
    } else {
        messageDisplay = "the file has been installed";
    }

    return (
        <li>
            <figure className="preview-pic">
                <i className="material-icons md-larger md-dark"><CloudDownload/></i>
            </figure>
            <div className="item-main">
                <h4>{item.filename}</h4>
                {progressBarDisplay}
                <p className="message">{messageDisplay}</p>
            </div>
        </li>
    )
}

function CollectionItemFile(props){

    const { ocsApiState, ocsApiDispatch } = React.useContext(Context);
    const [ isApplicableType, setIsApplicableType ] = useState(false);
    const [ isApplying, setIsApplying ] = useState(false);
    const [ isOpening, setIsOpening ] = useState(false);
    const [ isDeleting, setIsDeleting ] = useState(false);

    const item = props.item;
    const file = props.file;

    React.useEffect(() => { checkIfIsApplicable(); },[])
    React.useEffect(() => { checkIfIsApplicable(); },[props.item])

    function checkIfIsApplicable(){
        ocsApiState.ocsManagerApi.sendSync('DesktopThemeHandler::isApplicableType', [item.install_type]).then(function(res){
            setIsApplicableType(res.data[0]);
        })
    }

    function onApplyClick(filePath,installType){
        setIsApplying(true);
        setTimeout(() => {
            ocsApiState.ocsManagerApi.send('DesktopThemeHandler::applyTheme', [filePath, installType]).then(function(data){
                setIsApplying(false);
            })
        }, 1000);
    }

    function onOpenClick(fileUrl){
        setIsOpening(true);
        setTimeout(() => {
            ocsApiState.ocsManagerApi.send('SystemHandler::openUrl', [fileUrl]).then(function(data){
                setIsOpening(false);
            })
        }, 1000);
    }

    function onDeleteClick(itemUrl){
        setIsDeleting(true);
        setTimeout(() => {
            ocsApiState.ocsManagerApi.send('ItemHandler::uninstall', [itemUrl]);
            ipcRenderer.sendSync('previewpic', 'remove', itemUrl);
            const getAppConfigInstallTypes = ocsApiState.ocsManagerApi.sendSync('ConfigHandler::getAppConfigInstallTypes', [])
            const getUserConfigInstalledItems = ocsApiState.ocsManagerApi.sendSync('ConfigHandler::getUsrConfigInstalledItems', []);
            getAppConfigInstallTypes.then(function(res){
                ocsApiDispatch({type:'SET_INSTALL_TYPES',installTypes:res.data[0]});
                return getUserConfigInstalledItems;
            }).then(function(res){
                ocsApiDispatch({type:'SET_INSTALLED_ITEMS',installedItems:res.data[0]});
            });
            setIsDeleting(false);        
        });
    }

    const previewpicDirectory = ipcRenderer.sendSync('previewpic', 'directory');
    const previewpicUrl = 'file://' + previewpicDirectory + '/' + ( btoa(item.url).slice(-255) );
    const destination = ocsApiState.installTypes[item.install_type].destination;
    const filePath = destination + '/' + file;
    const fileUrl = 'file://'+filePath;

    let applyButtonDisplay;
    if (isApplicableType === true){
        
        let applyProgressBar;
        if (isApplying === true) applyProgressBar = <div id="progress-bar-container"><LinearProgress/></div>

        applyButtonDisplay = (
            <button onClick={e => onApplyClick(filePath,item.install_type)}>
                Apply
                {applyProgressBar}
            </button>
        )
    }

    let openProgressBar;
    if (isOpening === true) openProgressBar = <div id="progress-bar-container"><LinearProgress/></div>;

    let deleteProgressBar;
    if (isDeleting === true) deleteProgressBar = <div id="progress-bar-container"><LinearProgress/></div>;

    return (
        <li>
            <figure className="preview-pic" style={{"backgroundImage":"url('"+previewpicUrl+"')"}}></figure>
            <div className="item-main"><h4>{file}</h4></div>
            <nav className="actions">
                {applyButtonDisplay}
                <button onClick={e => onOpenClick(fileUrl)}>
                    {item.install_type === "bin" ? "Run" : "Open"}
                    {openProgressBar}
                </button>
                <button onClick={e => onDeleteClick(item.url)}>
                    Delete
                    {deleteProgressBar}
                </button>
            </nav>
        </li>      
    )
}

export default Collections;